.globl _start
@ Question: where do these values come from?
@ Answer: manuals for UART and memory map for board
@ See:
@ UART Manual (PrimeCell UART (PL011)), Section 3.2 and 3.3.1
@ http://infocenter.arm.com/help/topic/com.arm.doc.ddi0183f/DDI0183.pdf
@ Flag Register Offset: Table 3-1
@ Flag Register Bits: Table 3-4

@ versatilepb Board Manual, Section 4.1 and page 4-97
@ http://infocenter.arm.com/help/topic/com.arm.doc.dui0224i/DUI0224I_realview_platform_baseboard_for_arm926ej_s_ug.pdf

@ symbolic constants
.equ UART_ADDR,	0x101f1000	@ memory map UART register start
.equ OFFSET_FR, 0x018		@ flag register offset

.equ RXFE_BIT, 0x10		@ bit of flag register for receive status
.equ TXFF_BIT, 0x20		@ bit of flag register for transmit status

.equ COUNT, 0x10

_start:
	ldr r4,=UART_ADDR	@ r4 := 0x 101f 1000
	mov sp,#0x100000	@ set up stack

	ldr r1,=string
	bl print_string

	mov r1,#0		@ r1 := 0
	bl get_int		@ get integer from user

loop:	add r0,r0,r1 		@ r0 := r0 + r1
	add r1,r1,#1

	bl print_number		@ print number in r0

	cmp r1,#COUNT
	ble loop		@ goto loop: pc := loop

	infloop: b infloop	@ pc = infloop

string:	.asciz "Enter a number: "
.byte	0x00, 0x00, 0x00

@ print string starting at address in r1
print_string:
	push	{r0,r1,r2,lr}
str_out:
	ldr	r0,=UART_ADDR
	ldrb	r2,[r1]		@ r2 = mem[r1]
	cmp	r2,#0x00	@ '\0' = 0x00: null character?
	beq	str_done	@ if yes, quit
	str	r2,[r0]		@ otherwise, write char of string
	add	r1,r1,#1	@ go to next character
	b	str_out		@ repeat
str_done:
	pop	{r0,r1,r2,lr}
	bx	lr

@ r0 should be a buffer that can receive the data. 
@ The buffer should have size at least 1000 bytes.
get_int:
	@preamble
	push {r4, r5, r6, lr}

	@ main body
	ldr r4, =UART_ADDR
	mov r5, #0
	mov r6, #10

get_int_loop:
	bl get_char
	cmp r0, #'0'
	blt get_int_exit
	cmp r0, #'9'
	bgt get_int_exit

	sub r0, r0, #'0'
	mul r5, r6, r5
	add r5, r5, r0

	b get_int_loop

get_int_exit:
	mov r0, r5
	mov r5, #13
	str r5, [r4]
	mov r5, #10
	str r5, [r4]

	@ wrap-up
	pop {r4, r5, r6, lr}
	bx lr

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

get_char:
	@preamble
	push {r2, r3, r4, lr}

	@ main body
	ldr r4,=UART_ADDR		@ r4 := 0x 101f 1000

get_char_wait:
	ldr r2,[r4,#OFFSET_FR]		@ load IO flag register to r2
	and r3,r2,#RXFE_BIT		@ mask non receive fifo empty bits
	cmp r3, #0			@ check if r3 == 0
	bne get_char_wait		@ wait if not ready (if r3 != 0)

	ldr r0,[r4]
	str r0, [r4]

	@ wrap-up
	pop {r2, r3, r4, lr}
	bx lr

@ print digit in r0
print_digit:
	@ print_digit preamble
	push {r4,r5,lr}

	@ print_digit main body
	ldr r4,=UART_ADDR
	@ ASCII codes stored
	@ at [r4] get printed

	cmp r0, #10
	addlt r5, r0, #48
	addge r5, r0, #55
	str r5, [r4]

	@ print_digit wrap-up
	pop {r4,r5,lr}
	bx lr

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

@ print number in r0
print_number:
	@ print_number preamble
	push {r0,r4,r5,r6,r7,lr}

	@ print_number main body
	ldr r4,=UART_ADDR
	@ ASCII codes stored
	@ at [r4] get printed

	mov r5, #28
	mov r6, r0

print_number_loop:
	cmp r5, #0
	blt print_number_exit

	lsr r7, r6, r5
	and r7, r7, #0x0000000f

	mov r0, r7
	bl print_digit

	sub r5, r5, #4
	b print_number_loop

print_number_exit:
	@ print newline
	mov r5, #13
	str r5, [r4]
	mov r5, #10
	str r5, [r4]

	@ print_number wrap-up
	pop {r0,r4,r5,r6,r7,lr}
	bx lr

/*
mrs r0,cpsr		@ get CPSR into r0
bic r0, r0, #0x80	@ modify CPSR value in r0: r0 = r0 AND NOT( #0x80 )
msr cpsr_c, r0		@ update cpsr
swi #0x10
*/
